/* mpn_mod_1s_4p - from x86/fat directory for fat binary.
   Generated by configure - DO NOT EDIT. */

#define OPERATION_mod_1_4 1
#define __gmpn_mod_1s_4p           __gmpn_mod_1s_4p_fat
#define __gmpn_mod_1cs_4p          __gmpn_mod_1cs_4p_fat
#define __gmpn_preinv_mod_1s_4p  __gmpn_preinv_mod_1s_4p_fat
#define __gmpn_mod_1s_4p_cps     __gmpn_mod_1s_4p_cps_fat

#include "/mnt/share/cygpkgs/gmp/mingw64-i686/mingw64-i686-gmp.noarch/src/gmp-6.2.1/mpn/x86/fat/mod_1_4.c"

